#include <limits.h>
#include <math.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/times.h>
#include <unistd.h>

typedef short bool;
#define false 0
#define true  1

typedef unsigned char ubyte;
typedef unsigned int Pixel;
typedef unsigned long ulong;
typedef unsigned int uint;

int repFactorX=1, repFactorY=1;

typedef struct ImageGray ImageGray;

struct ImageGray
{
   long MinX;
   long MinY;
   ulong Width;
   ulong Height;
   Pixel *Pixmap;
};

ImageGray *ImageGrayCreate(long minx, long miny, ulong width, ulong height)
{
   ImageGray *img;

   img = malloc(sizeof(ImageGray));
   if (img==NULL)  return(NULL);
   img->MinX = minx;
   img->MinY = miny;
   img->Width = width;
   img->Height = height;
   img->Pixmap = malloc(width*height*sizeof(uint));
   if (img->Pixmap==NULL)
   {
      free(img);
      return(NULL);
   }
   return(img);
} /* ImageGrayCreate */

ImageGray *ImageGrayClone(ImageGray *in)
{
   ImageGray *img = ImageGrayCreate(in->MinX, 
				    in->MinY, 
				    in->Width, 
				    in->Height);

   int i, imagesize= in->Width * in->Height;
   if (img!=NULL){
     for (i=0; i<imagesize;i++)
       img->Pixmap[i] = in->Pixmap[i];
   }

   return(img);

} /* ImageClone */


void ImageGrayDelete(ImageGray *img)
{
   free(img->Pixmap);
   free(img);
} /* ImageGrayDelete */


ImageGray *ImageGrayPGMBinRead(char *fname)
{
   FILE *infile;
   ImageGray *img;
   ubyte *buffer;
   ulong width, height, maxval, p;
   int c, bpp=1;
   int x,y,rx,ry;

   infile = fopen(fname, "rb");
   if (infile==NULL)  return(NULL);
   fscanf(infile, "P5\n");
   while ((c=fgetc(infile)) == '#')
      while ((c=fgetc(infile)) != '\n');
   ungetc(c, infile);
   fscanf(infile, "%lu %lu\n%lu\n", &width, &height, &maxval);
   img = ImageGrayCreate(0, 0, width*repFactorX, height*repFactorY);
   if (img==NULL)
   {
      fclose(infile);
      return(NULL);
   }
   if (maxval>255)  bpp = 2;
   buffer = malloc(width*height*bpp);
   if (buffer==NULL)
   {
      ImageGrayDelete(img);
      fclose(infile);
      return(NULL);
   }
   fread(buffer, bpp, width*height, infile);
   fclose(infile);
   for (p=0,y=0;y<height;y++){
     for (x=0;x<width;x++,p++){
       img->Pixmap[y*width*repFactorX+x] = buffer[p*bpp];
     }
     for (rx=1;rx<repFactorX;rx++){
       for (x=0;x<width;x++)
	 img->Pixmap[y*width*repFactorX+x] = 
	   img->Pixmap[(y*repFactorX+rx)*width+x] =
	   img->Pixmap[(y*repFactorX+rx)*width-x-1];
     }
   }
   for (ry = 1; ry<repFactorY;ry++){
     for (y=0;y<height;y++){
       memcpy(img->Pixmap + (ry*height + y)*width*repFactorX,
	      img->Pixmap + (ry*height - y-1)*width*repFactorX,
	      width*repFactorX*sizeof(Pixel));
     }
   }
   free(buffer);
   return(img);
} /* ImageGrayPGMBinRead */



int ImageGrayPGMBinWrite(ImageGray *img, char *fname)
{
   FILE *outfile;
   ubyte *buffer;
   ulong p;

   outfile = fopen(fname, "wb");
   if (outfile==NULL)  return(-1);
   buffer = malloc((img->Width)*(img->Height));
   if (buffer==NULL)
   {
      fclose(outfile);
      return(-1);
   }
   for (p=0; p<(img->Width)*(img->Height); p++)
   {
      buffer[p] = img->Pixmap[p];
      //     buffer[p*2+1] = img->Pixmap[p] & 255;
   }
   fprintf(outfile, "P5\n%ld %ld\n%d\n", img->Width, img->Height, 255);
   fwrite(buffer, 1, (size_t)((img->Width)*(img->Height)), outfile);
   free(buffer);
   fclose(outfile);
   return(0);
} /* ImageGrayPGMBinWrite */
 
int main (int argc,char *argv[]){
  char *imgfname, *outfname;
  ImageGray *in;
  if (argc<=3){
    printf("Usage: %s <input file (pgm)> <output file (pgm)> <replication factor(X)> [replication factor Y]\n",argv[0]);
    printf("The last parameter is optional\n");
    exit(-1);
  }
  imgfname = argv[1];
  outfname = argv[2];
  repFactorX =  atoi(argv[3]);
  repFactorY = (argc<=4) ? repFactorX : atoi(argv[4]);
  in =ImageGrayPGMBinRead(imgfname);
  ImageGrayPGMBinWrite(in,outfname);
  return(0);
}
